<?php

namespace Teknisa\PLA\Modules\Consumo\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Method;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Setforms2\Shared\ORM\DateTime;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Exception;

class Pla00800PrevisaoDeConsumoController extends \Teknisa\GER\Controllers\TeknisaCrud  {
      private $entityManager;

      protected $dataSourceManager;
      protected $novoCodigo;
      protected $dataSourceName;
      protected $parameterBagColumns;
      protected $crudService;
      protected $environment;
      protected $birtStrategy;

      public function __construct($crudService, \Doctrine\ORM\EntityManager $entityManager, $dataSourceManager, $novoCodigo, $environment, $parameterBag) {
            $this->entityManager        = $entityManager;
            $this->dataSourceManager    = $dataSourceManager;
            $this->novoCodigo           = $novoCodigo;
            $this->environment          = $environment;
      parent::__construct($crudService, $parameterBag);
      }


      public function calculaPrevisao(DTO\Request\Row $request, DTO\Response $response) {

            try {
                  
                  ini_set('max_execution_time',500);
                  $row = $request->getRow();
                  $tipoPrevCons = $row['TIPOPREVCONSUMO'];
                  $nrlista = $this->setServico($row, 'NRLISTA');
                  $listafili = $this->setListaFili($row);
                  $dtinicial = $row['DTINICIAL'];
                  $dtfinal = $this->utilizaLimiteSeguranca($row);
                  $cdprodutoini = $row['CDPRODUTOINI'];
                  $cdprodutofin = $row['CDPRODUTOFIN'];
                  $cdoperador = $row['CDOPERADOR'];
                  $peacrescimo = $row['PEACRESCIMO'];
                  $idsepprato = $row['IDSEPPRATO'];
                  $idarredond = $row['IDARREDOND'];
                  $idprodutos = $row['IDPRODUTOS'];
                  $dthrcalcpc = $row['DTHRCALCPC'];
                  $nrconsplanth = $row['NRCONSPLANTH'];
                  $nrconsplan = "";
                  
                  if ($tipoPrevCons == "C") {

                        $nrconsplan = $this->CONSPLAN_GERAL($nrlista, $dtinicial, $dtfinal, $idprodutos, $idsepprato, $peacrescimo, $idarredond, $cdprodutoini, $cdprodutofin, $cdoperador, $nrconsplanth, $nrconsplan);
                        $nrconsplan = array($nrconsplan);
                        $response->addDataSet(new DataSet ('nrconsplan', $nrconsplan));

                  } else {

                        $this->PREV_CONS_LISTA($nrlista, $dtinicial, $dtfinal, $cdprodutoini, $cdprodutofin, $peacrescimo, $idarredond,  $cdoperador, $dthrcalcpc);
                  }

                  $this->excluiLista($nrlista);


            } catch(\Exception $e) {

                  $response->setError(new Error ($e->getMessage(), 500));
            }

      }

      public function CONSPLAN_GERAL($nrlista, $dtinicial, $dtfinal, $idprodutos, $idsepprato, $peacrescimo, $idarredond, $cdprodutoini, $cdprodutofin, $cdoperador, $nrconsplanth, $nrconsplan) {

            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection, 'CONSPLAN_GERAL');
            $procedure->addParam(new Param('NRLISTA', Param::PARAM_INPUT, $nrlista, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('DTINICIAL', Param::PARAM_INPUT, $dtinicial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('DTFINAL', Param::PARAM_INPUT, $dtfinal, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('IDPRODUTOS', Param::PARAM_INPUT, $idprodutos, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('IDSEPPRATO', Param::PARAM_INPUT, $idsepprato, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('PEACRESCIMO', Param::PARAM_INPUT, $peacrescimo, Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('IDARREDOND', Param::PARAM_INPUT, $idarredond, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDPRODUTOINI', Param::PARAM_INPUT, $cdprodutoini, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDPRODUTOFIN', Param::PARAM_INPUT, $cdprodutofin, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDOPERADOR', Param::PARAM_INPUT, $cdoperador, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('NRCONSPLANTH', Param::PARAM_INPUT, $nrconsplanth, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('NRCONSPLAN', Param::PARAM_OUTPUT, $nrconsplan, Param::PARAM_TYPE_STR, 10));
            $nrconsplan = $procedure->execute();
            return $nrconsplan['NRCONSPLAN'];
      }

      public function PREV_CONS_LISTA($nrlista, $dtinicial, $dtfinal, $cdprodutoini, $cdprodutofin, $peacrescimo, $idarredond,  $cdoperador, $dthrcalcpc) {

            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection, 'PREV_CONS_LISTA');
            $procedure->addParam(new Param('NRLISTA', Param::PARAM_INPUT, $nrlista, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('DTINICIAL', Param::PARAM_INPUT, $dtinicial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('DTFINAL', Param::PARAM_INPUT, $dtfinal, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDPRODUTOINI', Param::PARAM_INPUT, $cdprodutoini, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDPRODUTOFIN', Param::PARAM_INPUT, $cdprodutofin, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('PEACRESCIMO', Param::PARAM_INPUT, $peacrescimo, Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('IDARREDOND', Param::PARAM_INPUT, $idarredond, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('CDOPERADOR', Param::PARAM_INPUT, $cdoperador, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('DTHRCALCPC', Param::PARAM_INPUT, $dthrcalcpc, Param::PARAM_TYPE_STR));
            $procedure->execute();
      }

      public function utilizaLimiteSeguranca($row) {

           if ($row['LIMITESEGURANCA'] == "S") {

                  $dataLimite = $this->retornaLimiteSeguranca($row['LISTAFILI'], $row['DTFINAL'], $row['CDGRUPFILI']);
                  return $dataLimite[0]['LIMITE'];

           } else {

                  return $row['DTFINAL'];

           }
      }


      public function insereLista($rows) {
            
            $nrlista = $this->novoCodigoLista();
            foreach ($rows['CDLISTA'] as $row) {

                  $cdlista = $row['CDLISTA'];
                  $lista = new \Teknisa\Model\Geral\Entities\Lista();
                  $lista = $this->constroiLista($lista, $cdlista, $nrlista);
                  $this->entityManager->persist($lista);
                  $this->entityManager->flush();

            }

            return $nrlista;
      }

      public function excluiLista($nrlista) {

            $lista = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Lista')
            ->findBy(array("nrlista" => $nrlista));

            $this->entityManager->getConnection()->beginTransaction();

            foreach ($lista as $item) {
                  $this->entityManager->remove($item);
            }

            $this->entityManager->flush();
            $this->entityManager->getConnection()->commit();
      }

      public function novoCodigoLista() {

            $contador = 'LISTA';
            $nrorg = $this->environment->getNrOrg();
            $nrlista = $this->novoCodigo->getCodeByTableName($contador, $nrorg);
            return $nrlista;
      }

      public function constroiLista($lista, $cdlista, $nrlista) {

            $lista->setCdlista($cdlista);
            $lista->setNrlista($nrlista);
            $lista->setNrorg($this->environment->getNrOrg());
            $lista->setNrorgultatu($this->environment->getNrOrg());
            $lista->setNrorginclusao($this->environment->getNrOrg());
            $lista->setCdoperinclusao($this->environment->getCdOperador());
            $lista->setCdoperultatu($this->environment->getCdOperador());
            $lista->setDtinclusao(new DateTime());
            $lista->setDtultatu(new DateTime());
            $lista->setIdativo('S');
            return $lista;

      }

      public function toDateTime($date) {

            $explode = explode('/', $date);
            $newDate = new DateTime($explode[2].'-'.$explode[1].'-'.$explode[0]);
            return $newDate;
      }

      public function setDatas($row) {

            $row['DTINICIAL'] = $this->toDateTime($row['DTINICIAL'])->format('d/m/Y');
            $row['DTFINAL'] = $this->toDateTime($row['DTFINAL'])->format('d/m/Y');
            return $row;

      }

      public function concatenaFiliais($filiaisDoGrupfili) {

            $filiais = "_";

            foreach ($filiaisDoGrupfili as $filial) {

                  $filiais = $filiais . $filial['CDLISTA'] . "_";

            }

            return $filiais;
      }

      public function retornaLimiteSeguranca($cdfilial, $dtfinal, $cdgrupfili) {

            $FilterCriteria = new FilterCriteria('pla00800_limiteSeguranca');
            $FilterCriteria->addCondition('CDFILIAL', $cdfilial);
            $FilterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
            $FilterCriteria->addCondition('DTFIN', $dtfinal);
            $FilterCriteria->addCondition('CDGRUPFILI', $cdgrupfili);
            $this->dataSourceName = 'pla00800_limiteSeguranca';
            $this->parameterBagColumns = array('CDFILIAL', 'NRORG', 'DTFIN', 'CDGRUPFILI');
            $FilterCriteria = $this->crudService->populateParameterBag($FilterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($FilterCriteria);
            return $dataSet->getRows();
      }

      public function retornaServicos($cdfilial, $cdgrupfili) {

            $FilterCriteria = new FilterCriteria('pla00800_servico');
            $FilterCriteria->addCondition('CDFILIAL', $cdfilial);
            $FilterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
            $FilterCriteria->addCondition('CDOPERADOR', $this->environment->getCdOperador());
            $FilterCriteria->addCondition('CDGRUPFILI', $cdgrupfili);
            $this->dataSourceName = 'pla00800_servico';
            $this->parameterBagColumns = array('CDFILIAL', 'NRORG', 'CDOPERADOR', 'CDGRUPFILI');
            $FilterCriteria = $this->crudService->populateParameterBag($FilterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($FilterCriteria);
            return $dataSet->getRows();
      }

      public function retornaFiliais($cdgrupfili) {

            $FilterCriteria = new FilterCriteria('filial_operador_grupfili');
            $FilterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
            $FilterCriteria->addCondition('CDOPERADOR', $this->environment->getCdOperador());
            $FilterCriteria->addCondition('CDGRUPFILI', $cdgrupfili);
            $this->dataSourceName = 'filialgrupfili';
            $this->parameterBagColumns = array('NRORG', 'CDOPERADOR', 'CDGRUPFILI');
            $FilterCriteria = $this->crudService->populateParameterBag($FilterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($FilterCriteria);
            return $dataSet->getRows();
      }

      public function organizaServicos($servicos) {

            $servicosOrganizados = array();
            foreach ($servicos as $servico) {

                  $servicosOrganizados[] = array(
                        'CDLISTA'=>$servico['CDSERVICO']
                  );

            }

            return $servicosOrganizados;

      }

      public function organizaFiliais($filiais) {
            $filiaisOrganizadas = array();
            foreach ($filiais as $filial) {
                  $filiaisOrganizadas[] = array(
                        'CDLISTA' => isset($filial['CDFILIAL']) ? $filial['CDFILIAL'] : $filial['CDLISTA']
                  );
            }

            return $filiaisOrganizadas;

      }

      public function setServico($row, $field) {
            
            if ($row[$field] == "T") {

                  // buscar no banco todos os servicos
                  $servicos = $this->retornaServicos($row['LISTAFILI'], $row['CDGRUPFILI']);
                  $row['CDLISTA'] = $this->organizaServicos($servicos);
                  $row[$field] = $this->insereLista($row);
                  return $row[$field];

            } else {

                  $row['CDLISTA'] = $row[$field];
                  $row[$field] = $this->insereLista($row);
                  return $row[$field];
            }
      }

      public function setFilial($row, $field) {
            if ($row[$field] == "T") { // Todas as Filiais
                  // buscar no banco todas as filiais que o operador tem acesso
                  $filiais = $this->retornaFiliais($row['CDGRUPFILI']);
                  $filiais = $this->organizaFiliais($filiais);
                  $row['CDLISTA'] = $filiais;
                  $filiais = $this->concatenaFiliais($filiais);
                  return $filiais;

            } else {  // Apenas algumas filiais selecionadas
                  
                  $row['CDLISTA'] = $row[$field];
                  $filiais = $this->concatenaFiliais($row[$field]);
                  return $filiais;
            }
      }

      public function setFilialLista($row, $field) {

            if ($row[$field] == "T") { // Todas as Filiais

                  // buscar no banco todas as filiais que o operador tem acesso
                  $filiais = $this->retornaFiliais($row['CDGRUPFILI']);
                  $row['CDLISTA'] = $this->organizaFiliais($filiais);
                  $filiais = $this->insereLista($row);
                  //$filiais = $this->concatenaFiliais($filiais);
                  return $filiais;

            } else {  // Apenas algumas filiais selecionadas
                  $row['CDLISTA'] = $this->organizaFiliais($row[$field]);
                  $filiais = $this->insereLista($row);
                  //$row[$field] = $this->concatenaFiliais($row['CDLISTA']);
                  return $filiais;
            }
      }

      public function setListaFili($row) {
            if ($row['LISTAFILI'] == "T") { // tem apenas as filiais do Grupo de Filial selecionado ou Todas as filiais sem considerar grupfili
                  $filiais = $this->retornaFiliais($row['CDGRUPFILI']);
                  $filiaisDoGrupfili = $this->organizaFiliais($filiais);
                  $filiaisDoGrupfili = $this->concatenaFiliais($filiaisDoGrupfili);
                  return $filiaisDoGrupfili;

            } else {  // Apenas algumas filiais selecionadas

                  return $row['LISTAFILI'];
            }
      }
}